import 'dart:typed_data';

import 'package:dsb/dsb.dart' as dsb;
import 'package:dsbuntis/src/day.dart';
import 'package:dsbuntis/src/plan.dart';
import 'package:dsbuntis/src/sub.dart';
import 'package:html/dom.dart' as dom;
import 'package:html_search/html_search.dart';
import 'package:html_unescape/html_unescape.dart';

final _tag = RegExp(r'</?.+?>');
final _unescape = HtmlUnescape();
String _str(dom.Element e) =>
    _unescape.convert(e.innerHtml.replaceAll(_tag, '')).trim();

List<int> _parseIntsFromString(String s) {
  final out = <int>[];
  var lastindex = 0;
  for (var i = 0; i < s.length; i++) {
    if (!'0123456789'.contains(s[i])) {
      if (lastindex != i) out.add(int.parse(s.substring(lastindex, i)));
      lastindex = i + 1;
    }
  }
  out.add(int.parse(s.substring(lastindex, s.length)));
  return out;
}

typedef PlanParser = Substitution Function(int, List<String>);

class DownloadingPlan {
  String htmlUrl, previewUrl;
  String? id, dsbDate, dsbTitle;
  Future<String> html;
  Future<Uint8List>? preview;
  DownloadingPlan(this.htmlUrl, this.previewUrl, this.html, this.preview,
      this.id, this.dsbDate, this.dsbTitle);
}

extension UntisParsing on dsb.Session {
  // TODO: completely rework this
  Iterable<DownloadingPlan> downloadPlans(
    List json, {
    bool downloadPreviews = false,
  }) =>
      json
          .map((p) => p['Childs'])
          .reduce((v, e) => [...v, ...e])
          .where((x) => !x['Detail'].endsWith('.png'))
          .where((x) => !x['Detail'].endsWith('.jpg'))
          .map<DownloadingPlan>((p) => DownloadingPlan(
              p['Detail'],
              p['Preview'],
              http.get(p['Detail'],
                  ttl: Duration(days: 4), defaultCharset: String.fromCharCodes),
              downloadPreviews
                  ? http.getBin('$previewEndpoint/${p['Preview']}')
                  : null,
              p['Id'],
              p['Date'],
              p['Title']));
}

Iterable<Future<Plan?>> parsePlans(
  Iterable<DownloadingPlan> plans, {
  PlanParser parser = Substitution.fromUntis,
}) =>
    plans.map((p) async {
      final rawHtml = (await p.html)
          .replaceAll('\n', '')
          .replaceAll('\r', '')
          //just fyi: these regexes only work because there are no more newlines
          .replaceAll(RegExp(r'<h1.*?</h1>'), '')
          .replaceAll(RegExp(r'</?p.*?>'), '')
          .replaceAll(RegExp(r'<th.*?</th>'), '')
          .replaceAll(RegExp(r'<head.*?</head>'), '')
          .replaceAll(RegExp(r'<script.*?</script>'), '')
          .replaceAll(RegExp(r'<style.*?</style>'), '')
          .replaceAll(RegExp(r'</?html.*?>'), '')
          .replaceAll(RegExp(r'</?body.*?>'), '')
          .replaceAll(RegExp(r'</?font.*?>'), '')
          .replaceAll(RegExp(r'</?span.*?>'), '')
          .replaceAll(RegExp(r'</?center.*?>'), '')
          .replaceAll(RegExp(r'</?a.*?>'), '')
          .replaceAll(RegExp(r'<tr.*?>'), '<tr>')
          .replaceAll(RegExp(r'<td.*?>'), '<td>')
          .replaceAll(RegExp(r'<th.*?>'), '<th>')
          .replaceAll(RegExp(r' +'), ' ')
          .replaceAll(RegExp(r'<br />'), '')
          .replaceAll(RegExp(r'<!-- .*? -->'), '');
      try {
        var html = parse(rawHtml).first.children[1].children; //body
        final planTitle =
            searchFirst(html, (e) => e.className.contains('mon_title'))!
                .innerHtml;
        html = searchFirst(html, (e) => e.className.contains('mon_list'))!
            .children
            .first //for some reason <table>s like to contain <tbody>s
            .children;
        final subs = <Substitution>[];
        for (var i = 1; i < html.length; i++) {
          final e = html[i].children.map(_str).toList();
          final allLessons = e[1];
          for (final lesson in _parseIntsFromString(allLessons)) {
            final sub = parser(lesson, e);
            subs.add(sub);
          }
        }
        return Plan(matchDay(planTitle), subs, planTitle, p.htmlUrl,
            p.previewUrl, p.preview != null ? await p.preview : null);
      } catch (e) {
        return null;
      }
    });
