class Substitution extends Comparable {
  String affectedClass;
  int lesson;
  String? orgTeacher;
  String subTeacher;
  String subject;
  String notes;
  String? room;
  bool isFree;

  Substitution(
    this.affectedClass,
    this.lesson,
    this.subTeacher,
    this.subject,
    this.isFree, {
    this.notes = '',
    this.orgTeacher,
    this.room,
  });

  Substitution.raw(
    this.lesson, {
    required String affectedClass,
    required this.subTeacher,
    required this.subject,
    required this.notes,
    this.orgTeacher,
    this.room,
  })  : affectedClass = affectedClass[0] == '0'
            ? affectedClass.substring(1).toLowerCase()
            : affectedClass.toLowerCase(),
        isFree = subTeacher.contains('---');

  static Substitution fromUntis(int lesson, List<String> e) =>
      e.length < 6 ? fromUntis2020(lesson, e) : fromUntis2021(lesson, e);

  static Substitution fromUntis2021(int lesson, List<String> e) =>
      Substitution.raw(
        lesson,
        affectedClass: e[0],
        subTeacher: e[2],
        subject: e[3],
        notes: e[5],
        orgTeacher: e[4],
      );

  static Substitution fromUntis2020(int lesson, List<String> e) =>
      Substitution.raw(
        lesson,
        affectedClass: e[0],
        subTeacher: e[2],
        subject: e[3],
        notes: e[4],
      );

  static Substitution fromUntis2019(int lesson, List<String> e) =>
      Substitution.raw(
        lesson,
        affectedClass: e[0],
        subject: e[2],
        subTeacher: e[3],
        orgTeacher: e[4],
        room: e[5],
        notes: e[6],
      );

  static String? _ng(j, String k) => j.containsKey(k) ? j[k] : null;

  Substitution.fromJson(dynamic json)
      : affectedClass = json['class'],
        lesson = json['lesson'],
        subTeacher = json['sub_teacher'],
        orgTeacher = _ng(json, 'org_teacher'),
        subject = json['subject'],
        notes = json['notes'],
        isFree = json['free'],
        room = _ng(json, 'room');

  Map<String, dynamic> toJson() {
    final m = <String, dynamic>{
      'class': affectedClass,
      'lesson': lesson,
      'sub_teacher': subTeacher,
      'subject': subject,
      'notes': notes,
      'free': isFree,
    };
    if (orgTeacher != null) m['org_teacher'] = orgTeacher;
    if (room != null) m['room'] = room;
    return m;
  }

  @override
  int compareTo(dynamic other) {
    if (other is int) {
      return lesson.compareTo(other);
    } else if (other is Substitution) {
      var tc = affectedClass, oc = other.affectedClass;
      if (tc.length > 1 && !'0123456789'.contains(tc[1])) tc = '0' + tc;
      if (oc.length > 1 && !'0123456789'.contains(oc[1])) oc = '0' + oc;
      final c = tc.compareTo(oc);
      return c != 0 ? c : lesson.compareTo(other.lesson);
    } else {
      return 0;
    }
  }

  @override
  String toString() =>
      "['$affectedClass', $lesson, '$orgTeacher' → '$subTeacher' at '$room', '$subject', '$notes', $isFree]";
}
